from typing import Dict, List, Union

from django import template
from django.core.handlers.wsgi import WSGIRequest
from django.template.base import Node
from django.template.exceptions import TemplateSyntaxError
from django.template.loader_tags import construct_relative_path
from django.utils.safestring import SafeString, mark_safe

from coltrane.renderer import (
    get_html_and_markdown,
    render_html_with_django,
    render_markdown_path,
    render_markdown_text,
)
from coltrane.retriever import get_content_directory, get_content_paths


register = template.Library()


class NoParentError(Exception):
    pass


@register.simple_tag(takes_context=True)
def directory_contents(
    context, directory: str = None, exclude: str = None
) -> List[Dict[str, str]]:
    if not directory:
        request = context["request"]
        directory = request.path
    elif isinstance(directory, SafeString):
        # Force SafeString to be a normal string so it can be used with `Path` later
        directory = directory + ""

    if directory and directory.startswith("/"):
        directory = directory[1:]

    content_paths = get_content_paths(directory)
    contents = []

    for path in content_paths:
        if path.name != "index.md":
            path_slug = path.name[:-3]
            content_slug = path_slug
            content_directory = get_content_directory()

            if directory:
                content_slug = f"{directory}/{path_slug}"
                content_directory = content_directory / directory

            if exclude:
                if exclude.startswith("/"):
                    exclude = exclude[1:]

                if exclude == content_slug:
                    continue

            (_, metadata) = get_html_and_markdown(content_slug)

            contents.append(metadata)

    return contents


@register.filter()
def parent(path: Union[str, WSGIRequest] = "") -> str:
    if isinstance(path, WSGIRequest) or hasattr(path, "path"):
        # Handle if a `request` is passed in
        path = path.path

    path = path.strip()

    if path.endswith("/"):
        path = path[:-1]

    if path == "":
        raise NoParentError()

    last_slash_index = path.rindex("/")
    path = path[:last_slash_index]

    return path


class IncludeMarkdownNode(Node):
    """
    Based on: `django.template.loader_tags.IncludeNode`.
    """

    context_key = "__include_markdown_context"

    def __init__(self, template, *args, **kwargs):
        self.template = template
        super().__init__(*args, **kwargs)

    def render(self, context):
        """
        Render the specified template and context.
        """

        template_name = self.template.resolve(context)

        cache = context.render_context.dicts[0].setdefault(self, {})
        template = cache.get(template_name)

        if template is None:
            template = context.template.engine.select_template((template_name,))
            cache[template_name] = template

        (html, metadata) = render_markdown_path(template.origin.name)

        for c in context:
            for key, value in c.items():
                if key not in metadata:
                    metadata[key] = value

        return render_html_with_django(html, metadata)


@register.tag("include_md")
def do_include_md(parser, token):
    """
    Load a markdown template and render it with the current context.

    Based on: `django.template.loader_tags.do_include`.

    Example:
        {% include "foo/some_include" %}
    """
    bits = token.split_contents()

    if len(bits) < 2:
        raise TemplateSyntaxError(
            "%r tag takes at least one argument: the name of the template to "
            "be included." % bits[0]
        )

    bits[1] = construct_relative_path(parser.origin.template_name, bits[1])

    return IncludeMarkdownNode(
        parser.compile_filter(bits[1]),
    )


@register.filter(takes_context=True)
def to_html(context: dict, text: str) -> str:
    (html, metadata) = render_markdown_text(text)

    return mark_safe(
        render_html_with_django(html, metadata, request=context["request"])
    )



import logging
import sys
from copy import deepcopy
from os import getenv
from pathlib import Path
from typing import Any, Dict, List, Optional

from django import setup as django_setup
from django.conf import settings
from django.core.handlers.wsgi import WSGIHandler
from django.template.library import InvalidTemplateLibrary, import_library

from dotenv import load_dotenv

from coltrane.config.settings import DEFAULT_COLTRANE_SETTINGS

from .utils import dict_merge


logger = logging.getLogger(__name__)

__all__ = [
    "initialize",
]


DEFAULT_CACHES_SETTINGS = {
    "default": {
        "BACKEND": "django.core.cache.backends.dummy.DummyCache",
    }
}

DEFAULT_MIDDLEWARE_SETTINGS = [
    "django.middleware.security.SecurityMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]


DEFAULT_INSTALLED_APPS = [
    "django.contrib.humanize",
    "django.contrib.sitemaps",
    "django.contrib.staticfiles",
    "django_fastdev",
    "coltrane",
]


def _get_base_dir(base_dir: Optional[Path]) -> Path:
    """
    Gets the base directory.
    """

    if base_dir is None:
        base_dir = Path(".")
    elif isinstance(base_dir, str):
        base_dir = Path(base_dir)

    return base_dir


def _get_template_tag_module_name(base_dir: Path, file: Path) -> str:
    """
    Get a dot notation module name if a particular file path is a template tag.
    """

    # TODO: Cleaner way to convert a string path to a module dot notation?
    module_name = str(file)

    if str(base_dir) != ".":
        module_name = module_name.replace(str(base_dir), "")

    module_name = module_name.replace("/", ".")

    if module_name.startswith("."):
        module_name = module_name[1:]

    if module_name.endswith(".py"):
        module_name = module_name[:-3]
    else:
        raise InvalidTemplateLibrary()

    import_library(module_name)

    return module_name


def _get_default_template_settings(base_dir: Path) -> List:
    """
    Gets default template settings, including templates and built-in template tags.
    """

    template_dir = base_dir / "templates"
    templatetags_dir = base_dir / "templatetags"
    template_tags = []

    if templatetags_dir.exists():
        for template_tag_path in templatetags_dir.rglob("*.py"):
            if template_tag_path.is_file():
                try:
                    module_name = _get_template_tag_module_name(
                        base_dir, template_tag_path
                    )
                    template_tags.append(module_name)
                except InvalidTemplateLibrary as e:
                    logger.debug(e)

    builtins = [
        "django.contrib.humanize.templatetags.humanize",
        "django.templatetags.static",
        "coltrane.templatetags.coltrane_tags",
    ]
    builtins.extend(template_tags)

    return [
        {
            "BACKEND": "django.template.backends.django.DjangoTemplates",
            "APP_DIRS": True,
            "DIRS": [template_dir],
            "OPTIONS": {
                "builtins": builtins,
                "context_processors": [
                    "django.template.context_processors.request",
                    "django.template.context_processors.debug",
                    "django.template.context_processors.static",
                ],
            },
        }
    ]


def _merge_installed_apps(
    django_settings: Dict[str, Any], installed_apps: List[str]
) -> List[str]:
    """
    Gets the installed apps from the passed-in settings and adds `coltrane` to it.
    """

    if "INSTALLED_APPS" in django_settings:
        installed_apps.extend(list(django_settings["INSTALLED_APPS"]))

    return installed_apps


def _get_from_env(env_name: str) -> List[str]:
    """
    Retrieves environment value that could potentially be a list of strings.
    """

    env_values = []

    if value_from_env := getenv(env_name):
        env_values = value_from_env.split(",")

    return env_values


def _is_whitenoise_installed() -> bool:
    """
    Helper function to check if `whitenoise` is installed.
    """

    try:
        import whitenoise  # type: ignore

        return True
    except ModuleNotFoundError:
        pass

    return False


def _merge_settings(base_dir: Path, django_settings: Dict[str, Any]) -> Dict[str, Any]:
    """
    Merges the passed-in settings into the default `coltrane` settings.
    Passed-in settings will override the defaults.
    """

    # Assume that `argv[1] == "build"` means that the `build`
    # management command is currently being run
    is_build_management_command = len(sys.argv) >= 2 and sys.argv[1] == "build"

    debug = django_settings.get("DEBUG", getenv("DEBUG", "True") == "True")

    staticfiles_dirs = [
        base_dir / "static",
    ]

    middleware = deepcopy(DEFAULT_MIDDLEWARE_SETTINGS)
    installed_apps = deepcopy(DEFAULT_INSTALLED_APPS)

    is_whitenoise_installed = _is_whitenoise_installed()

    if is_whitenoise_installed:
        middleware.insert(1, "whitenoise.middleware.WhiteNoiseMiddleware")
        installed_apps.insert(0, "whitenoise.runserver_nostatic")

    if debug and not is_build_management_command:
        # Add settings required for django-browser-reload when appropriate
        middleware.append("django_browser_reload.middleware.BrowserReloadMiddleware")
        installed_apps.append("django_browser_reload")

        # Add content and data to "staticfiles" so django-browser-reload will monitor
        staticfiles_dirs.append(base_dir / "content")
        staticfiles_dirs.append(base_dir / "data")

    default_settings = {
        "BASE_DIR": base_dir,
        "ROOT_URLCONF": "coltrane.urls",
        "DEBUG": debug,
        "SECRET_KEY": getenv("SECRET_KEY"),
        "INSTALLED_APPS": _merge_installed_apps(django_settings, installed_apps),
        "CACHES": DEFAULT_CACHES_SETTINGS,
        "MIDDLEWARE": middleware,
        "TEMPLATES": _get_default_template_settings(base_dir),
        "INTERNAL_IPS": _get_from_env("INTERNAL_IPS"),
        "ALLOWED_HOSTS": _get_from_env("ALLOWED_HOSTS"),
        "STATIC_ROOT": base_dir / "output" / "static",
        "STATIC_URL": "static/",
        "STATICFILES_DIRS": staticfiles_dirs,
        "LOGGING": {
            "version": 1,
            "disable_existing_loggers": False,
            "handlers": {
                "console": {
                    "class": "logging.StreamHandler",
                },
            },
            "root": {
                "handlers": ["console"],
                "level": "ERROR",
            },
        },
        "COLTRANE": DEFAULT_COLTRANE_SETTINGS,
        # Hard-coding `SETTINGS_MODULE` even though it kind of doesn't make sense, but
        # is needed for rendering a nice 500 page on local when debugging
        "SETTINGS_MODULE": "coltrane",
    }

    coltrane_site_url = getenv("COLTRANE_SITE_URL")

    if coltrane_site_url:
        default_settings["COLTRANE"]["SITE_URL"] = coltrane_site_url

    coltrane_title = getenv("COLTRANE_TITLE")

    if coltrane_title:
        default_settings["COLTRANE"]["TITLE"] = coltrane_title

    coltrane_description = getenv("COLTRANE_DESCRIPTION")

    if coltrane_description:
        default_settings["COLTRANE"]["DESCRIPTION"] = coltrane_description

    if is_whitenoise_installed:
        default_settings["WHITENOISE_MANIFEST_STRICT"] = False
        default_settings[
            "STATICFILES_STORAGE"
        ] = "whitenoise.storage.CompressedManifestStaticFilesStorage"

    # Make sure BASE_DIR is a `Path` if it got passed in
    if "BASE_DIR" in django_settings and isinstance(django_settings["BASE_DIR"], str):
        django_settings["BASE_DIR"] = Path(django_settings["BASE_DIR"])

    # Override STATIC_ROOT if the output directory name is manually set
    try:
        django_settings["STATIC_ROOT"] = (
            base_dir / django_settings["COLTRANE"]["OUTPUT"]["PATH"] / "static"
        )
    except KeyError:
        pass

    # Override STATIC_ROOT if the output directory is manually set
    try:
        django_settings["STATIC_ROOT"] = (
            Path(django_settings["COLTRANE"]["OUTPUT"]["DIRECTORY"]) / "static"
        )
    except KeyError:
        pass

    django_settings = dict_merge(
        default_settings, django_settings, destination_overrides_source=True
    )
    logger.debug(f"Merged settings: {django_settings}")

    return django_settings


def _configure_settings(django_settings: Dict[str, Any]) -> None:
    """
    Configures the settings in Django.
    """

    settings.configure(**django_settings)


def initialize(
    **django_settings: Dict[str, Any],
) -> WSGIHandler:
    """
    Initializes the Django static site.
    """

    base_dir = _get_base_dir(django_settings.get("BASE_DIR"))

    load_dotenv(base_dir / ".env")

    django_settings = _merge_settings(base_dir, django_settings)
    _configure_settings(django_settings)

    django_setup()

    return WSGIHandler()



import logging
from typing import Dict, Tuple

from django.http import Http404, HttpRequest
from django.shortcuts import render
from django.utils.cache import patch_response_headers

from .config.cache import ViewCache
from .renderer import render_markdown


logger = logging.getLogger(__name__)


def _normalize_slug(slug: str) -> str:
    if slug is None:
        slug = ""

    if slug.endswith("/"):
        slug = slug[:-1]

    if slug.startswith("/"):
        slug = slug[1:]

    if slug == "":
        slug = "index"

    return slug


def _get_from_cache_if_enabled(slug: str) -> Tuple[str, Dict]:
    """
    Gets the slug from the cache if it's enabled.
    """

    template: str = None
    context: Dict = None
    view_cache = ViewCache()

    if view_cache.is_enabled:
        cache_key = f"{view_cache.cache_key_namespace}{slug}"
        cached_value = view_cache.cache.get(cache_key)

        if cached_value:
            (template, context) = cached_value

    return (template, context)


def _set_in_cache_if_enabled(slug: str, template: str, context: Dict) -> None:
    """
    Sets everything in the cache if it's enabled.
    """

    view_cache = ViewCache()

    if view_cache.is_enabled:
        cache_key = f"{view_cache.cache_key_namespace}{slug}"
        view_cache.cache.set(
            cache_key,
            (template, context),
            view_cache.seconds,
        )


def content(request: HttpRequest, slug: str = "index"):
    """
    Renders the markdown file stored in `content` based on the slug from the URL.
    Adds data into the context from `data.json` and JSON files in the `data` directory.
    Will cache the rendered content if enabled.
    """

    template = ""
    context: Dict = {}
    slug = _normalize_slug(slug)

    (template, context) = _get_from_cache_if_enabled(slug)

    try:
        if not template or not context:
            (template, context) = render_markdown(slug, request=request)
            _set_in_cache_if_enabled(slug, template, context)
    except FileNotFoundError:
        try:
            slug_with_index = f"{slug}/index"
            (template, context) = render_markdown(slug_with_index, request=request)
            _set_in_cache_if_enabled(slug_with_index, template, context)
        except FileNotFoundError:
            raise Http404(f"{slug} cannot be found")

    response = render(
        request,
        template,
        context=context,
    )

    view_cache = ViewCache()

    if view_cache.is_enabled:
        patch_response_headers(response, cache_timeout=view_cache.seconds)

    return response



import codecs
import logging
import re
from dataclasses import dataclass, field
from typing import Dict, Optional, Tuple, Union

from django.http import HttpRequest
from django.template import engines
from django.utils.html import mark_safe  # type: ignore
from django.utils.timezone import now

import dateparser
from markdown2 import Markdown, markdown

from .config.paths import get_content_directory
from .config.settings import get_markdown_extras, get_site_url
from .retriever import get_data


logger = logging.getLogger(__name__)

DEFAULT_TEMPLATE = "coltrane/content.html"


@dataclass
class StaticRequest(HttpRequest):
    """
    Used to mock an HttpRequest when generating the HTML for static sites.

    Required for `coltrane.templatetags.coltrane.current_direct
    """

    path: str
    META: Dict = field(default_factory=dict)
    GET: Dict = field(default_factory=dict)

    def __init__(self, path: str, meta=None, get=None):
        self.path = path
        self.META = meta or {}
        self.GET = get or {}
    
    @property
    def site_url(self):
        site_url = get_site_url()
        assert site_url, "COLTRANE_SITE_URL in .env or COLTRANE.SITE_URL in settings file is required"

        return site_url

    @property
    def scheme(self) -> str:
        return str(self.site_url.split("://")[0])

    def get_host(self) -> str:
        return str(self.site_url.split("://")[1])

    def is_secure(self) -> bool:
        return self.path.startswith("https://")


def _parse_and_update_metadata(content: Markdown) -> dict:
    """
    Add new, parse and/or cast existing values to metadata.
    """

    metadata = content.metadata

    if metadata is None:
        metadata = {}

    if "publish_date" in metadata:
        metadata["publish_date"] = dateparser.parse(metadata["publish_date"])

    if "draft" in metadata:
        metadata["draft"] = metadata["draft"] == "true"

    metadata["now"] = now()

    if hasattr(content, "toc_html"):
        metadata["toc"] = mark_safe(content.toc_html)

    return metadata


def render_markdown_path(path) -> Tuple[str, Dict]:
    """
    Renders the markdown file located at path.
    """

    with codecs.open(path, "r", encoding="utf-8") as f:
        text = f.read()

        return render_markdown_text(text)


def render_markdown_text(text: str) -> Tuple[str, Dict]:
    markdown_extras = get_markdown_extras()

    # Wrap code fences with Django `verbatim` templatetag
    text = re.sub(pattern=r"```.*?```", repl="{% verbatim %}\n\g<0>\n{% endverbatim %}", string=text, flags=re.RegexFlag.DOTALL)

    content = markdown(
        text=text,
        extras=markdown_extras
    )

    metadata = _parse_and_update_metadata(content)

    # Remove `p` tags that get added to the `verbatim` templatetag
    content = content.replace("<p>{% verbatim %}</p>\n", "{% verbatim %}")
    content = content.replace("\n<p>{% endverbatim %}</p>", "{% endverbatim %}")

    return (str(content), metadata)


def _get_markdown_content_as_html(slug: str) -> Tuple[str, Optional[Dict]]:
    """
    Converts markdown file based on the slug into HTML.
    """

    path = get_content_directory() / f"{slug}.md"

    return render_markdown_path(path)


def render_html_with_django(
    html: str, context: Dict, request: HttpRequest = None
) -> str:
    """
    Takes the rendered HTML from the markdown uses Django to fill in any template
    variables from the `context` dictionary.
    """

    django_engine = engines["django"]
    template = django_engine.from_string(html)

    return str(template.render(context=context, request=request))


def get_html_and_markdown(slug: str) -> Tuple[str, Dict]:
    (html, metadata) = _get_markdown_content_as_html(slug)

    if metadata is None:
        metadata = {}

    if "template" not in metadata:
        metadata["template"] = DEFAULT_TEMPLATE

    metadata["slug"] = slug

    return (html, metadata)


def render_markdown(
    slug: str, request: Union[HttpRequest, StaticRequest]
) -> Tuple[str, Dict]:
    """
    Renders the markdown from the `slug` by:
    1. Rendering the markdown file into HTML
    2. Passing the HTML through Django to fill in template variables based on
        data in JSON files and markdown frontmatter

    Returns:
        Tuple of template file name (i.e. `coltrane/content.html`) and context dictionary.
    """

    (html, metadata) = get_html_and_markdown(slug)

    context = {}

    # Start with any metadata from the markdown frontmatter
    context.update(metadata)

    # Add JSON data to the context
    data = get_data()
    context["data"] = data

    if request:
        context["request"] = request

    # Add rendered content to the context
    content = render_html_with_django(html, context, request)
    context["content"] = mark_safe(content)
    template = context["template"]

    return (template, context)



"""
A little wrapper CLI over a few Django management commands.
"""

from os import getcwd
from pathlib import Path
from stat import S_IEXEC
from subprocess import run as run_process

from django.core.management.utils import get_random_secret_key

import click
import rich_click  # type: ignore


click.Command.format_help = rich_click.rich_format_help  # type: ignore
click.Group.format_help = rich_click.rich_format_help  # type: ignore


DEFAULT_APP = """#!/usr/bin/env python

from django.core.management import execute_from_command_line

from coltrane import initialize


wsgi = initialize()

if __name__ == "__main__":
    execute_from_command_line()
"""

DEFAULT_ENV = """DEBUG=True
INTERNAL_IPS=127.0.0.1
ALLOWED_HOSTS=
COLTRANE_SITE_URL=
SECRET_KEY="""

DEFAULT_WATCHMAN_CONFIG = """{
  "ignore_dirs": ["node_modules"]
}"""


def _run_management_command(command_name, *args):
    current_directory = getcwd()
    file_path = Path(current_directory) / "app.py"

    run_process(
        [
            file_path,
            command_name,
        ]
        + list(args)
    )


class AliasedGroup(click.Group):
    def get_command(self, ctx, cmd_name):
        try:
            cmd_name = ALIASES[cmd_name].name
        except KeyError:
            pass
        return super().get_command(ctx, cmd_name)


@click.group(cls=AliasedGroup, help="Runs commands for coltrane.")
def cli():
    pass


@click.command(help="Create a new coltrane site.")
@click.option("--force/--no-force", default=False, help="Force creating a new site")
def create(force):
    app_file = Path("app.py")

    if app_file.exists() and not force:
        click.secho("Skip project creation because app.py already exists.", fg="red")
    else:
        click.secho("Start creating files...\n", fg="yellow")
        Path("__init__.py").touch()

        click.secho("- Create app.py")
        app_file.write_text(DEFAULT_APP)

        click.secho("- Set app.py as executable")
        app_file.chmod(app_file.stat().st_mode | S_IEXEC)

        click.secho("- Create .env")
        Path(".env").write_text(DEFAULT_ENV + get_random_secret_key())

        click.secho("- Create .watchmanconfig")
        Path(".watchmanconfig").write_text(DEFAULT_WATCHMAN_CONFIG)

        click.secho("- Create content directory")
        Path("content").mkdir(exist_ok=True)
        (Path("content") / "index.md").write_text("# index.md")

        click.secho("- Create data directory")
        Path("data").mkdir(exist_ok=True)

    click.secho()
    click.secho("For local development: ", nl=False, fg="green")
    click.secho("poetry run coltrane play")
    click.secho("Build static HTML: ", nl=False, fg="green")
    click.secho("poetry run coltrane record")


@click.command(help="Start a local development server.")
@click.option("--port", default=8000, help="Port to serve localhost from")
def play(port):
    _run_management_command("runserver", f"0:{port}")


@click.command(help="Generates HTML output.")
@click.option("--force/--no-force", default=False, help="Force HTML generation")
@click.option(
    "--threads", type=int, help="Number of threads to use when generating static files"
)
@click.option("--output", help="Output directory")
@click.option("--ignore/--no-ignore", default=False, help="Ignore errors")
def record(force, threads, output, ignore):
    args = []

    if force:
        args.append("--force")

    if output:
        args.append("--output")
        args.append(output)

    if threads:
        args.append("--threads")
        args.append(str(threads))

    if ignore:
        args.append("--ignore")

    _run_management_command("build", *args)


cli.add_command(create)
cli.add_command(play)
cli.add_command(record)


ALIASES = {
    "init": create,
    "serve": play,
    "build": record,
    "rec": record,
}



from typing import Dict, Optional

from django.conf import settings


# List of all available extras: https://github.com/trentm/python-markdown2/wiki/Extras
DEFAULT_MARKDOWN_EXTRAS = [
    "fenced-code-blocks",
    "header-ids",
    "metadata",
    "strike",
    "tables",
    "task_list",
    "nofollow",
    "code-friendly",
    "footnotes",
    "numbering",
    "strike",
    "toc",
]

DEFAULT_COLTRANE_SETTINGS = {"MARKDOWN_EXTRAS": DEFAULT_MARKDOWN_EXTRAS}


def get_coltrane_settings() -> Dict:
    """
    Get the `COLTRANE` settings from the settings file.
    """

    if hasattr(settings, "COLTRANE"):
        assert isinstance(
            settings.COLTRANE, dict
        ), "COLTRANE settings should be a dictionary"

        return settings.COLTRANE

    return DEFAULT_COLTRANE_SETTINGS


def get_markdown_extras() -> int:
    """
    Get the markdown extras.
    """

    return get_coltrane_settings().get("MARKDOWN_EXTRAS", DEFAULT_MARKDOWN_EXTRAS)


def get_site_url() -> Optional[str]:
    """
    Get the configured site.
    """

    return get_coltrane_settings().get("SITE_URL")


def get_title() -> Optional[str]:
    """
    Get the configured title.
    """

    return get_coltrane_settings().get("TITLE")


def get_description() -> Optional[str]:
    """
    Get the configured description.
    """

    return get_coltrane_settings().get("DESCRIPTION")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['coltrane',
 'coltrane.config',
 'coltrane.management',
 'coltrane.management.commands',
 'coltrane.templatetags']

package_data = \
{'': ['*'], 'coltrane': ['templates/coltrane/*']}

install_requires = \
['Django>3.0',
 'click>=8.0.0,<9.0.0',
 'dateparser>=1.1.0,<2.0.0',
 'django-browser-reload>=1.3.0,<2.0.0',
 'django-fastdev>=1.3.0,<2.0.0',
 'halo>=0.0.31,<0.0.32',
 'markdown2>=2.4.2,<3.0.0',
 'python-dotenv>0.17',
 'rich-click>=0.2.0,<0.3.0']

extras_require = \
{'deploy': ['gunicorn>=20.1.0,<21.0.0', 'whitenoise>=5.3.0,<6.0.0'],
 'docs': ['Sphinx>=4.3.2,<5.0.0',
          'linkify-it-py>=1.0.3,<2.0.0',
          'myst-parser>=0.16.1,<0.17.0',
          'furo>=2021.11.23,<2022.0.0',
          'sphinx-copybutton>=0.4.0,<0.5.0',
          'sphinx-autobuild>=2021.3.14,<2022.0.0',
          'toml',
          'attrs>=21.4.0,<22.0.0']}

entry_points = \
{'console_scripts': ['coltrane = coltrane.console:cli']}

setup_kwargs = {
    'name': 'coltrane',
    'version': '0.20.0',
    'description': 'A simple content site framework that harnesses the power of Django without the hassle.',
    'long_description': '<p align="center">\n  <a href="https://coltrane.readthedocs.io"><h1 align="center">coltrane</h1></a>\n</p>\n<p align="center">A simple content site framework that harnesses the power of Django without the hassle 🎵</p>\n\n![PyPI](https://img.shields.io/pypi/v/coltrane?color=blue&style=flat-square)\n![PyPI - Downloads](https://img.shields.io/pypi/dm/coltrane?color=blue&style=flat-square)\n![GitHub Sponsors](https://img.shields.io/github/sponsors/adamghill?color=blue&style=flat-square)\n<!-- ALL-CONTRIBUTORS-BADGE:START - Do not remove or modify this section -->\n![All Contributors](https://img.shields.io/badge/all_contributors-1-orange.svg?style=flat-square)\n<!-- ALL-CONTRIBUTORS-BADGE:END -->\n\n📖 Complete documentation: https://coltrane.readthedocs.io\n\n📦 Package located at https://pypi.org/project/coltrane/\n\n## ⭐ Features\n\n- Can either generate a static HTML site, be deployed as a standalone Django site, or integrated into an existing Django site\n- Reads markdown content and renders it in HTML\n- Can use data from JSON files in templates and markdown content\n- Automatic generation of `sitemap.xml` and `rss.xml` files\n- [Live re-rendering of markdown and data](https://twitter.com/adamghill/status/1487522925393715205) when markdown or JSON data files are saved with the magic of https://github.com/adamchainz/django-browser-reload\n- All the power of Django templates, template tags, and filters inside markdown files\n- Can include other Django apps for additional functionality\n- Custom Template tags are supported and are enabled automatically for use in markdown content\n- Opinionated standalone Django project setup where deployment (including static files) just works "out of the box"\n\n## ⚡ Quick start for a new static site\n\n1. `mkdir new-site && cd new-site` to create a new folder\n1. `poetry init --no-interaction --dependency \'coltrane:<1\' && poetry install` to create a new virtual environment and install the `coltrane` package\n1. `poetry run coltrane create` to create the folder structure for a new site\n1. Update `content/index.md`\n1. `poetry run coltrane play` for a local development server\n1. Go to http://localhost:8000 to see the updated markdown rendered into HTML\n1. `poetry run coltrane record` to output the rendered HTML files\n\n### Optional installation\n\n- Enable `watchman` for less resource-intensive autoreload on MacOS: `brew install watchman`\n\n## ➕ How to add new content\n\nAdd markdown files or sub-directories with markdown files to the `content` directory and they will automatically have routes created that can be requested.\n\n**Example markdown files**\n\n```\ncontent/index.md\ncontent/about.md\ncontent/articles/this-is-the-first-article.md\n```\n\n**`poetry run coltrane play` will serve these URLs**\n\n- `http://localhost:8000/` which serves HTML generated from the `/content/index.md` file\n- `http://localhost:8000/about/` which serves HTML generated from the `/content/about.md` file\n- `http://localhost:8000/articles/this-is-the-first-article/` which serves HTML generated from the `/content/articles/this-is-the-first-article.md` file\n- `http://localhost:8000/not-there/` will 404\n\n**`poetry run coltrane record` will create these HTML files for a static site**\n\n- `output/index.html`\n- `output/about/index.html`\n- `output/articles/this-is-the-first-article/index.html`\n\nRead all of the documentation at https://coltrane.readthedocs.io.\n\n## Contributors ✨\n\nThanks goes to these wonderful people ([emoji key](https://allcontributors.org/docs/en/emoji-key)):\n\n<!-- ALL-CONTRIBUTORS-LIST:START - Do not remove or modify this section -->\n<!-- prettier-ignore-start -->\n<!-- markdownlint-disable -->\n<table>\n  <tbody>\n    <tr>\n      <td align="center"><a href="https://github.com/Tobi-De"><img src="https://avatars.githubusercontent.com/u/40334729?v=4?s=100" width="100px;" alt="Tobi DEGNON"/><br /><sub><b>Tobi DEGNON</b></sub></a><br /><a href="https://github.com/adamghill/coltrane/commits?author=Tobi-De" title="Tests">⚠️</a> <a href="https://github.com/adamghill/coltrane/commits?author=Tobi-De" title="Code">💻</a></td>\n    </tr>\n  </tbody>\n  <tfoot>\n    \n  </tfoot>\n</table>\n\n<!-- markdownlint-restore -->\n<!-- prettier-ignore-end -->\n\n<!-- ALL-CONTRIBUTORS-LIST:END -->\n\nThis project follows the [all-contributors](https://github.com/all-contributors/all-contributors) specification. Contributions of any kind welcome!\n',
    'author': 'adamghill',
    'author_email': 'adam@adamghill.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/adamghill/coltrane/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from django.conf import settings
from django.contrib.sitemaps.views import sitemap
from django.urls import include, path, re_path

from coltrane.feeds import ContentFeed
from coltrane.sitemaps import ContentSitemap

from . import views


app_name = "coltrane"

sitemaps = {"content": ContentSitemap}

urlpatterns = []

if settings.DEBUG:
    urlpatterns += [
        path("__reload__/", include("django_browser_reload.urls")),
    ]

urlpatterns += [
    path(
        "sitemap.xml",
        sitemap,
        {"sitemaps": sitemaps},
        name="django.contrib.sitemaps.views.sitemap",
    ),
    path("rss.xml", ContentFeed()),
    re_path(r"^(?P<slug>(\w|-|\/)*)", views.content, name="content"),
]
